"""
RTRogTools.py

    Tool library for RealTimeROG.py.
    
    Copyright (c) 2011 University of Toronto
    Original Version :  12 August 2011 by Donald J. Woodbury 
    Contact : David Bailey <dbailey@physics.utoronto.ca>
                (www.physics.utoronto.ca/~dbailey)
    License: Released under the MIT License; the full terms are appended
                to the end of this file, and are also available
                at www.opensource.org/licenses/mit-license.php
"""

from Tkinter import *
from tkSimpleDialog import askstring
from tkFileDialog import asksaveasfilename, askopenfilename
from numpy import array, zeros, nonzero, reshape, argwhere, loadtxt
from ImageDraw import Draw
from PIL import Image, ImageTk, ImageDraw
import os
import pylab

class Calibration:
    '''Launches a Toplevel window where the user is prompted to click on the
    four sides of the plate. Once all four sides are selected, another window
    opens prompting the user to input the diameter of the plate. The ratio of
    pixels to centimeters is then saved and the positions of each of the sides
    is saved under the names self.pix_to_cm and self.corners.'''
    
    def __init__(self, master, image):

        self.master = master
        self.window = Toplevel(master)
        self.window.title("Calibration")
        
        self.resize_factor = 400.0/image.size[1]

        image = image.resize((int(image.size[0]*self.resize_factor), 400))
        
        self.corners = []
        self.pix_to_cm = [1,1]

        size = image.size
        self.title = StringVar()
        self.title.set('Select left side of plate:')
        label = Label(self.window, textvariable=self.title)
        label.pack()
        frame = Frame(self.window)
        frame.pack()
        canvas = Canvas(frame, width=size[0], height=size[1])
        canvas.pack()
        self.tkimg = ImageTk.PhotoImage(image, master = self.window)
        display_image = canvas.create_image(0,0,image=self.tkimg, \
                                                 anchor = 'nw')
        
        canvas.bind("<Button-1>", self.return_pos)

        self.i = 0
        
    def return_pos(self, event):
        '''Saves the positions of the sides of the plates and computes the
        conversion from pixels to centimeters.'''
        self.corners.append((event.x/self.resize_factor, event.y/self.resize_factor))

        self.i+=1

        if self.i == 1:
            self.title.set('Select top of plate:')
        if self.i == 2:
            self.title.set('Select right side of plate:')
        if self.i == 3:
            self.title.set('Select bottom of plate:')
        if self.i == 4:           
            self.window.destroy()
            try:
                self.diameter = float(askstring('Plate Diameter',\
                                            'Diameter of the Plate (cm):\n(Note'+\
                                            ' that \'Cancel\' will set the\nunits'+\
                                            ' of the ROG to pixels)',\
                                            initialvalue = '20', parent = self.master))
                self.pix_to_cm[:] = [self.diameter/(self.corners[2][0]-self.corners[0][0]),\
                                     self.diameter/(self.corners[3][1]-self.corners[1][1])]
            except TypeError:
                pass

class Plot_ROG:
    '''Launches a Toplevel window and prompts the user to select a file
    containing the ROG data saved by the Save ROG function and then to
    write titles and labels for a plot. A pylab plot will then be displayed. '''
    def __init__(self, master):

        self.filename = askopenfilename(parent=master, \
                                   filetypes = [('Text File','*.txt')],\
                                   title="Choose ROG Data...")

        if len(self.filename) > 0:
            self.Plot_params(master)

    def Plot_params(self, master):
        '''Prompts the user to select the titles and lables for the plot.'''
        
        self.window = Toplevel(master)
        self.window.title("Plot Labels")

        self.title = StringVar()
        self.title.set('Radius of Gyration over time')
        self.x_label = StringVar()
        self.x_label.set('Time (seconds)')
        self.y_label = StringVar()
        self.y_label.set('Radius of Gyration (cm)')
        
        L1 = Label(self.window, text='Plot Title: ')
        L1.grid(row = 0, sticky = 'w')
        L2 = Label(self.window, text='x Label: ')
        L2.grid(row = 1, sticky = 'w')
        L3 = Label(self.window, text='y Label: ')
        L3.grid(row = 2, sticky = 'w')

        ask_title = Entry(self.window, textvariable=self.title, width= 40)
        ask_title.grid(row = 0, column = 1, columnspan = 3, )
        ask_x_label = Entry(self.window, textvariable=self.x_label, width= 40)
        ask_x_label.grid(row = 1, column = 1, columnspan = 3)
        ask_y_label = Entry(self.window, textvariable=self.y_label, width= 40)
        ask_y_label.grid(row = 2, column = 1, columnspan = 3)
        
        ask_ok = Button(self.window, text='Plot', command = self.Plot_graph)
        ask_ok.grid(row = 3, column = 1)
        ask_cancel = Button(self.window, text='Cancel', command = self.End_plot)
        ask_cancel.grid(row = 3, column = 2)

        self.window.protocol("WM_DELETE_WINDOW", self.End_plot)
        
    def Plot_graph(self):
        '''launches a pylab plot of the RoG.'''
        self.window.destroy()

        data = loadtxt(self.filename, skiprows = 2)

        x = data[:,0]
        y = data[:,1]
        
        pylab.plot(x, y)
        pylab.xlabel(self.x_label.get())
        pylab.ylabel(self.y_label.get())
        pylab.title(self.title.get())

        pylab.show()
    def End_plot(self):
        self.window.destroy()  

def draw_points(points, image):
    '''Returns a copy of the image 'image' with each point in the list 'points'
    drawn as a red pixel drawn on the image.'''

    im = image.copy().convert('RGB')
    draw = ImageDraw.Draw(im)

    for point in points:
        draw.point(point, fill = 'Red')
    
    return im

def points_above_threshold(image, threshold, bbox):
    '''Returns a list of the pixel indicies of all of the pixels in the image
    whose value exceeds the given threshold. If the list 'centers' in non-empty
    then only a square of radius box_size around each point in centers will
    be searched. If centers is empty, or index is divisible by 10, the entire
    area defined by bbox will be searched. Requires 'array' to be a 2D
    bitmap array of numbers (for our purposes these numbers represent an
    8-bit image and therefore take on values from 0-255.)'''

    points = []
    
    if len(bbox) == 4:

        pix =  image.load()
        
        (a, b) = ((bbox[2][0]-bbox[0][0])/2.0, (bbox[3][1]-bbox[1][1])/2.0)
        (x0, y0) = ((bbox[2][0]+bbox[0][0])/2.0, (bbox[3][1]+bbox[1][1])/2.0)
        
        for y in xrange(int(bbox[1][1]), int(bbox[3][1])):
            r = (abs(-((y-y0)*a/b)**2+a**2)) ** 0.5
            for x in xrange(int(x0-r), int(x0+r)):
                if pix[x,y] > threshold:
                    points.append((x, y))
                    
    else:
        image = image.point(lambda p: p > threshold)
        
        im_buffer = array(image.getdata()).astype('float')
        im_array = reshape(im_buffer, (image.size[1], image.size[0]))
        
        try:
            B = argwhere(im_array)
            (ystart, xstart), (ystop, xstop) = B.min(0), B.max(0) + 1 
            cropped_array = im_array[ystart:ystop, xstart:xstop]
            
            p = array(nonzero(cropped_array)).swapaxes(0,1)

            for (y,x) in p:
                points.append((xstart+x,ystart+y))
        except ValueError:
            pass

    return points

def find_center(tups):
    '''Take in a list of 2-tuples. Find the center of the coordinates those
       2-tuples represent and return the center as a 2-tuple of floats
    '''

    x, y = 0, 0
    for t in tups:
        x, y = x + t[0], y + t[1]
    x, y = float(x) / len(tups), float(y) / len(tups)

    return (x, y)

def radius_of_gyration(centers, (x_pix_to_cm, y_pix_to_cm)):
    '''Returns the radius of gyration given the list of tuple bead
    locations in centers'''

    (x0, y0) = find_center(centers)

    RoG = 0

    for (x, y) in centers:
        RoG += (x_pix_to_cm*(x-x0))**2+(y_pix_to_cm*(y-y0))**2

    RoG = (RoG/len(centers))**0.5

    return RoG

def Resize_image(image, xsize, ysize):
    '''returns a copy of the image scaled to fit completly in a box of size
    (xsize, ysize). preserves the x:y ratio and makes either the length or
    height of the final image equal xsize or ysize respectively.'''
        
    x = image.size[0]*ysize/image.size[1]
    y = image.size[1]*xsize/image.size[0]
    if y < ysize:
        scaled_image = image.copy().resize((xsize, y))
    else:
        scaled_image = image.copy().resize((x, ysize))

    return scaled_image
